@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2002, 2003, 2006 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,7 +28,6 @@
 
 #define _(x) gettext(x)
 
-
 static void
 do_new_event (GtkWidget *widget, gpointer user_data)
 {
@@ -49,6 +48,7 @@ do_new_event (GtkWidget *widget, gpointer user_data)
 static void
 destroy_popup (GtkWidget *widget, GtkWidget *parent)
 {
+  gtk_grab_remove (widget);
   g_object_set_data (G_OBJECT (parent), "popup-handle", NULL);
   pop_window = NULL;
 }
@@ -95,21 +95,82 @@ static gboolean
 key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *window)
 {
   if (k->keyval == GDK_Escape) 
-  {
-    gtk_widget_destroy(window);
-    return TRUE;
-  }
+    {
+      gtk_widget_destroy(window);
+      return TRUE;
+    }
   return FALSE;
 }
 
+static gboolean
+handle_expose_event (GtkWidget *w, GdkEventExpose *ev, GtkWidget *child)
+{
+  gdk_window_clear (w->window);
+
+  gdk_draw_rectangle (w->window, w->style->black_gc, FALSE, 0, 0, 
+		      w->allocation.width - 1, w->allocation.height - 1);
+  
+  /* Draw the contents */
+  gtk_container_propagate_expose (GTK_CONTAINER (w), child, ev);
+
+  return TRUE;
+}
+
+static gboolean
+handle_configure_event (GtkWidget *window, GdkEventConfigure *ev, gpointer user_data)
+{
+  gint x, y;
+  gint screen_width, screen_height;
+
+  /* Adjust the window's position to try to keep it on the screen.  */
+  x = (gint)g_object_get_data (G_OBJECT (window), "x");
+  y = (gint)g_object_get_data (G_OBJECT (window), "y");
+
+  screen_width = gdk_screen_width ();
+  screen_height = gdk_screen_height ();
+
+  x = CLAMP (x, 0, MAX (0, screen_width - ev->width));
+  y = CLAMP (y, 0, MAX (0, screen_height - ev->height));
+
+  gtk_widget_set_uposition (window, MAX (x, 0), MAX (y, 0));
+
+  return FALSE;
+}
+
+static gint
+button_press_event (GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+  GtkWidget *child;
+
+  child = gtk_get_event_widget (event);
+
+  /* We don't ask for button press events on the grab widget, so
+   *  if an event is reported directly to the grab widget, it must
+   *  be on a window outside the application (and thus we remove
+   *  the popup window). Otherwise, we check if the widget is a child
+   *  of the grab widget, and only remove the popup window if it
+   *  is not.
+   */
+  if (child != widget)
+    {
+      while (child)
+	{
+	  if (child == widget)
+	    return FALSE;
+
+	  child = child->parent;
+	}
+    }
+  
+  gtk_widget_destroy (widget);
+  
+  return TRUE;
+}
 
 GtkWidget *
 day_popup (GtkWidget *parent, struct day_popup *p, gboolean show_items)
 {
-  GtkRequisition requisition;
   gint x, y;
-  gint screen_width;
-  gint screen_height;
   GtkWidget *window = gtk_window_new (GTK_WINDOW_POPUP);
   GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
   GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
@@ -118,8 +179,6 @@ day_popup (GtkWidget *parent, struct day_popup *p, gboolean show_items)
   GtkWidget *new_button = NULL;
   GtkWidget *label;
   GtkWidget *contents = NULL;
-  GtkWidget *frame;
-  char buf[256];
   struct tm tm;
   gchar *gs;
 
@@ -132,9 +191,8 @@ day_popup (GtkWidget *parent, struct day_popup *p, gboolean show_items)
   tm.tm_mon = p->month;
   tm.tm_mday = p->day;
   mktime (&tm);
-  strftime (buf, sizeof (buf), "%a %d %B", &tm);
-
-  gs = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+  
+  gs = strftime_strdup_utf8_locale ("%a %d %B", &tm);
   label = gtk_label_new (gs);
   g_free (gs);
   
@@ -153,53 +211,49 @@ day_popup (GtkWidget *parent, struct day_popup *p, gboolean show_items)
           gtk_clist_clear (GTK_CLIST (contents));
       
           while (events)
-        {
-          GdkPixmap *pmap;
-          GdkBitmap *bmap;
-          
-          event_t ev = events->data;
-          event_details_t evd = event_db_get_details (ev);
-          char *p = buf;
-          size_t s = sizeof (buf), l;
-          gchar *lineinfo[1];
-    
-          localtime_r (&ev->start, &tm);
-          l = strftime (p, s, TIMEFMT, &tm);
-          s -= l;
-          p += l;
-    
-          snprintf (p, s - 1, " %s", evd->summary);
-          p[s - 1] = 0;
-          
-          lineinfo[0] = buf;
-          
-          gtk_clist_append (GTK_CLIST (contents), lineinfo);
-    
-          gtk_clist_set_row_data (GTK_CLIST (contents), row, ev);
-          
-          if ((ev->flags & FLAG_ALARM) && ev->recur)
-            {
-              if (gpe_find_icon_pixmap ("bell_recur", &pmap, &bmap))
-            gtk_clist_set_pixtext (GTK_CLIST (contents), row, 0, buf, 5,
-                           pmap, bmap);
-            }
-          else if (ev->flags & FLAG_ALARM)
-            { 
-              if (gpe_find_icon_pixmap ("bell", &pmap, &bmap))
-            gtk_clist_set_pixtext (GTK_CLIST (contents), row, 0, buf, 5,
-                           pmap, bmap);
-            }
-          else if (ev->recur)
-            {
-              if (gpe_find_icon_pixmap ("recur", &pmap, &bmap))
-            gtk_clist_set_pixtext (GTK_CLIST (contents), row, 0, buf, 5,
-                           pmap, bmap);
-            }
-            
-          row++;
-          events = events->next;
-        }
-        
+	    {
+	      GdkPixmap *pmap;
+	      GdkBitmap *bmap;
+	      event_t ev = events->data;
+	      event_details_t evd = event_db_get_details (ev);
+	      gchar *lineinfo[1];
+	      gchar *timestr;
+
+	      localtime_r (&ev->start, &tm);
+	      timestr = strftime_strdup_utf8_locale (TIMEFMT, &tm);
+	      lineinfo[0] = g_strdup_printf ("%s %s", timestr, evd->summary);
+	      
+	      gtk_clist_append (GTK_CLIST (contents), lineinfo);
+
+	      g_free (timestr);
+	      
+	      gtk_clist_set_row_data (GTK_CLIST (contents), row, ev);
+	      
+	      if ((ev->flags & FLAG_ALARM) && ev->recur)
+		{
+		  if (gpe_find_icon_pixmap ("bell_recur", &pmap, &bmap))
+		    gtk_clist_set_pixtext (GTK_CLIST (contents), row, 0, lineinfo[0], 5,
+					   pmap, bmap);
+		}
+	      else if (ev->flags & FLAG_ALARM)
+		{ 
+		  if (gpe_find_icon_pixmap ("bell", &pmap, &bmap))
+		    gtk_clist_set_pixtext (GTK_CLIST (contents), row, 0, lineinfo[0], 5,
+					   pmap, bmap);
+		}
+	      else if (ev->recur)
+		{
+		  if (gpe_find_icon_pixmap ("recur", &pmap, &bmap))
+		    gtk_clist_set_pixtext (GTK_CLIST (contents), row, 0, lineinfo[0], 5,
+					   pmap, bmap);
+		}
+	      
+	      g_free (lineinfo[0]);
+
+	      row++;
+	      events = events->next;
+	    }
+	  
           gtk_clist_thaw (GTK_CLIST (contents));
           g_signal_connect (G_OBJECT (contents), "select_row",
                             G_CALLBACK (selection_made), window);
@@ -209,14 +263,12 @@ day_popup (GtkWidget *parent, struct day_popup *p, gboolean show_items)
           contents = gtk_label_new (_("No appointments"));
         }
     }
-  else  /* !show_items */
-    {
-      new_button = 
-        gpe_button_new_from_stock (GTK_STOCK_NEW, GPE_BUTTON_TYPE_BOTH);
-      gtk_button_set_relief (GTK_BUTTON (new_button), GTK_RELIEF_NONE);
-      g_signal_connect (G_OBJECT (new_button), "clicked",
+
+  new_button = 
+    gpe_button_new_from_stock (GTK_STOCK_NEW, GPE_BUTTON_TYPE_BOTH);
+  gtk_button_set_relief (GTK_BUTTON (new_button), GTK_RELIEF_NONE);
+  g_signal_connect (G_OBJECT (new_button), "clicked",
                     G_CALLBACK (do_new_event), p);
-    }
 
   gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
   gtk_button_set_relief (GTK_BUTTON (day_button), GTK_RELIEF_NONE);
@@ -240,35 +292,34 @@ day_popup (GtkWidget *parent, struct day_popup *p, gboolean show_items)
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   if (contents)
     gtk_box_pack_start (GTK_BOX (vbox), contents, TRUE, TRUE, 0);
-  if (new_button)
-    gtk_box_pack_start (GTK_BOX (vbox), new_button, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), new_button, TRUE, TRUE, 0);
 
-  frame = gtk_frame_new (NULL);
-  gtk_container_add (GTK_CONTAINER (frame), vbox);
-  gtk_container_add (GTK_CONTAINER (window), frame);
+  gtk_container_add (GTK_CONTAINER (window), vbox);
 
   gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (parent));
 
   gdk_window_get_pointer (NULL, &x, &y, NULL);
 
-  gtk_widget_show_all (frame);
+  x -= 2;
+  y += 4;
 
-  gtk_widget_realize (window);
-  gtk_widget_size_request (window, &requisition);
-  
-  screen_width = gdk_screen_width ();
-  screen_height = gdk_screen_height ();
+  g_object_set_data (G_OBJECT (window), "x", (gpointer)x);
+  g_object_set_data (G_OBJECT (window), "y", (gpointer)y);
 
-  x = CLAMP (x - 2, 0, MAX (0, screen_width - requisition.width));
-  y = CLAMP (y + 4, 0, MAX (0, screen_height - requisition.height));
-  
-  gtk_widget_set_uposition (window, MAX (x, 0), MAX (y, 0));
+  gtk_widget_set_uposition (window, x, y);
 
   g_object_set_data (G_OBJECT (parent), "popup-handle", window);
-  g_signal_connect (G_OBJECT (window), "destroy",
-                    G_CALLBACK (destroy_popup), parent);
+  g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (destroy_popup), parent);
+  g_signal_connect (G_OBJECT (window), "configure_event", G_CALLBACK (handle_configure_event), NULL);
+  g_signal_connect (G_OBJECT (window), "expose_event", G_CALLBACK (handle_expose_event), vbox);
+  g_signal_connect (G_OBJECT (window), "button_press_event", G_CALLBACK (button_press_event), NULL);
 
-  gtk_widget_show (window);
+  gtk_widget_show_all (window);
+
+  gdk_pointer_grab (window->window, TRUE,
+		    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK,
+		    NULL, NULL, gtk_get_current_event_time ());
+  gtk_grab_add (window);
 
   return window;
 }
