@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001, 2002, 2003, 2004 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2001, 2002, 2003, 2004, 2006 Philip Blundell <philb@gnu.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,11 +19,22 @@
 
 #include <time.h>
 #include <gtk/gtk.h>
+#include <string.h>
 #include "gtkdatesel.h"
 #include <libintl.h>
 
+#include "globals.h"
+
 #define _(x) dgettext(PACKAGE, x)
 
+#ifdef IS_HILDON
+  #define LABEL_ADD  16
+  #define ARROW_SIZE 30
+#else
+  #define LABEL_ADD   8
+  #define ARROW_SIZE 20
+#endif
+
 static guint my_signals[1];
 
 struct elem
@@ -56,31 +67,6 @@ struct _GtkDateSelClass
 
 static GtkHBoxClass *parent_class = NULL;
 
-static int
-days_in_month (struct tm *tm)
-{
-  if (tm->tm_mon == 1)
-    {
-      guint y = 1900 + tm->tm_year;
-      if ((y % 4) == 0
-          && (((y % 100) != 0) || (y % 400) == 0))
-        return 29;
-      else
-        return 28;
-    }
-
-  switch (tm->tm_mon)
-    {
-    case 3:
-    case 5:
-    case 8:
-    case 10:
-      return 30;
-    default:
-      return 31;
-    }
-}
-
 void
 gtk_date_sel_move_day (GtkDateSel *sel, int d)
 {
@@ -136,7 +122,7 @@ gtk_date_sel_move_month (GtkDateSel *sel, int d)
           tm.tm_mon = 0;
         }
     }
-  mdays = days_in_month (&tm);
+  mdays = days_in_month (tm.tm_year + 1900, tm.tm_mon);
   if (tm.tm_mday > mdays)
     {
       sel->day_clamped = tm.tm_mday;
@@ -170,7 +156,7 @@ year_click (GtkWidget *w, GtkDateSel *sel)
   guint mdays;
   localtime_r (&sel->time, &tm);
   tm.tm_year += d;
-  mdays = days_in_month (&tm);
+  mdays = days_in_month (tm.tm_year + 1900, tm.tm_mon);
   if (tm.tm_mday > mdays)
     {
       sel->day_clamped = tm.tm_mday;
@@ -193,20 +179,14 @@ static void
 format_text (time_t *time, GtkWidget *w, char *fmt)
 {
   struct tm tm;
-  char buf[64];
-  gchar *sbuf, *sfmt;
+  gchar *sbuf;
   localtime_r (time, &tm);
-  sfmt = g_locale_from_utf8 (fmt, -1, NULL, NULL, NULL);
-  if (sfmt)
+
+  sbuf = strftime_strdup_utf8_utf8 (fmt, &tm);
+  if (sbuf)
     {
-      strftime (buf, sizeof (buf), sfmt, &tm);
-      sbuf = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-      if (sbuf)
-	{
-	  gtk_label_set_text (GTK_LABEL (w), sbuf);
-	  g_free (sbuf);
-	}
-      g_free (sfmt);
+      gtk_label_set_text (GTK_LABEL (w), sbuf);
+      g_free (sbuf);
     }
 }
 
@@ -248,6 +228,98 @@ day_update (GtkDateSel *sel, GtkWidget *w)
   format_text (&sel->time, w, _("%a, %d"));
 }
 
+static int
+get_max_month_width (GtkDateSel *sel)
+{
+  PangoLayout *layout;
+  PangoRectangle logical_rect;
+  int max_width, i;
+  struct tm tm;
+
+  localtime_r (&sel->time, &tm);
+
+  layout = gtk_widget_create_pango_layout (GTK_WIDGET (sel), NULL);
+
+  max_width = 0;
+  for (i = 0; i < 11; i++)
+    {
+      gchar *str = NULL;
+
+      tm.tm_mon = i;
+
+      switch (sel->month_style)
+	{
+	case GTKDATESEL_MONTH_SHORT:
+	  str = strftime_strdup_utf8_utf8 (_("%b"), &tm);
+	  break;
+	case GTKDATESEL_MONTH_LONG:
+	  str = strftime_strdup_utf8_utf8 (_("%B"), &tm);
+	  break;
+	case GTKDATESEL_MONTH_NUMERIC:
+	  str = strftime_strdup_utf8_utf8 (_("%m"), &tm);
+	  break;
+	case GTKDATESEL_MONTH_ROMAN:
+	  /* NYS */
+	  break;
+	}
+
+      pango_layout_set_text (layout, str, -1);
+      g_free (str);
+
+      pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+      max_width = MAX (max_width, logical_rect.width + LABEL_ADD);
+    }
+
+  g_object_unref (layout);
+
+  return max_width;
+}
+
+static int
+get_max_day_width (GtkDateSel *sel)
+{
+  PangoLayout *layout;
+  PangoRectangle logical_rect;
+  int max_width, i, w;
+  struct tm tm;
+  gchar *str;
+
+  localtime_r (&sel->time, &tm);
+
+  layout = gtk_widget_create_pango_layout (GTK_WIDGET (sel), NULL);
+
+  max_width = 0;
+  for (i = 0; i < 31; i++)
+    {
+      tm.tm_mday = i;
+
+      str = strftime_strdup_utf8_utf8 (_("%d"), &tm);
+      pango_layout_set_text (layout, str, -1);
+      g_free (str);
+
+      pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+      max_width = MAX (max_width, logical_rect.width + LABEL_ADD);
+    }
+  w = max_width;
+  
+  max_width = 0;
+  for (i = 0; i < 7; i++)
+    {
+      tm.tm_wday = i;
+
+      str = strftime_strdup_utf8_utf8 (_("%a"), &tm);
+      pango_layout_set_text (layout, str, -1);
+      g_free (str);
+
+      pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+      max_width = MAX (max_width, logical_rect.width + LABEL_ADD);
+    }
+
+  g_object_unref (layout);
+
+  return w + max_width;
+}
+
 static void
 make_field (GtkDateSel *sel, struct elem *e, 
 	    void (*click)(GtkWidget *, GtkDateSel *),
@@ -255,17 +327,20 @@ make_field (GtkDateSel *sel, struct elem *e,
 	    int max_width)
 {
   GtkWidget *arrow_l, *arrow_r, *arrow_button_l, *arrow_button_r;
-
+#ifdef IS_HILDON
+  arrow_l = gtk_image_new_from_stock(GTK_STOCK_GO_BACK, GTK_ICON_SIZE_SMALL_TOOLBAR);
+  arrow_r = gtk_image_new_from_stock(GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_SMALL_TOOLBAR);
+#else
   arrow_l = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_OUT);
   arrow_r = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
-
+#endif
   arrow_button_l = gtk_button_new ();
   arrow_button_r = gtk_button_new ();
   e->text = gtk_label_new ("");
   e->container = gtk_hbox_new (FALSE, 0);
 
-  gtk_widget_set_size_request (arrow_button_l, 20, 20);
-  gtk_widget_set_size_request (arrow_button_r, 20, 20);
+  gtk_widget_set_size_request (arrow_button_l, ARROW_SIZE, ARROW_SIZE);
+  gtk_widget_set_size_request (arrow_button_r, ARROW_SIZE, ARROW_SIZE);
   GTK_WIDGET_UNSET_FLAGS (arrow_button_l, GTK_CAN_FOCUS);
   GTK_WIDGET_UNSET_FLAGS (arrow_button_r, GTK_CAN_FOCUS);
 
@@ -298,7 +373,7 @@ gtk_date_sel_init (GtkDateSel *sel)
 {
   PangoLayout *layout;
   PangoRectangle logical_rect;
-  int max_year_width, i;
+  int max_year_width, i, max_day_width;
   gchar buffer[255];
 
   layout = gtk_widget_create_pango_layout (GTK_WIDGET (sel), NULL);
@@ -314,11 +389,13 @@ gtk_date_sel_init (GtkDateSel *sel)
       g_snprintf (buffer, sizeof (buffer), "%d%d%d%d", i,i,i,i);
       pango_layout_set_text (layout, buffer, -1);	  
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
-      max_year_width = MAX (max_year_width, logical_rect.width + 8);
+      max_year_width = MAX (max_year_width, logical_rect.width + LABEL_ADD);
     }
 
-  make_field (sel, &sel->day, day_click, day_update, 72);
-  make_field (sel, &sel->week, week_click, week_update, 72);
+  max_day_width = get_max_day_width (sel);
+
+  make_field (sel, &sel->week, week_click, week_update, 0);
+  make_field (sel, &sel->day, day_click, day_update, max_day_width);
   make_field (sel, &sel->month, month_click, month_update, 0);
   make_field (sel, &sel->year, year_click, year_update, max_year_width);
 
@@ -456,42 +533,7 @@ gtk_date_sel_set_day(GtkDateSel *sel, int year, int month, int day)
 void
 gtk_date_sel_set_month_style (GtkDateSel *sel, GtkDateSelMonthStyle style)
 {
-  PangoLayout *layout;
-  PangoRectangle logical_rect;
-  int max_width, i;
-  gchar buffer[255];
-  struct tm tm;
-
-  localtime_r (&sel->time, &tm);
-
-  layout = gtk_widget_create_pango_layout (GTK_WIDGET (sel), NULL);
-
-  max_width = 0;
-  for (i = 0; i < 11; i++)
-    {
-      tm.tm_mon = i;
-
-      switch (style)
-	{
-	case GTKDATESEL_MONTH_SHORT:
-	  strftime (buffer, sizeof (buffer), _("%b"), &tm);
-	  break;
-	case GTKDATESEL_MONTH_LONG:
-	  strftime (buffer, sizeof (buffer), _("%B"), &tm);
-	  break;
-	case GTKDATESEL_MONTH_NUMERIC:
-	  strftime (buffer, sizeof (buffer), _("%m"), &tm);
-	  break;
-	case GTKDATESEL_MONTH_ROMAN:
-	  /* NYS */
-	  break;
-	}
-      pango_layout_set_text (layout, buffer, -1);
-      pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
-      max_width = MAX (max_width, logical_rect.width + 8);
-    }
-
-  gtk_widget_set_usize (sel->month.text, max_width, -1);
   sel->month_style = style;
-  g_object_unref (layout);
+
+  gtk_widget_set_usize (sel->month.text, get_max_month_width (sel), -1);
 }
