@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2001, 2002, 2003 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2001, 2002, 2003, 2006 Philip Blundell <philb@gnu.org>
  * Hildon adaption 2005 by Matthias Steinbauer <matthias@steinbauer.org>
+ * Toolbar new API conversion 2005 by Florian Boor <florian@kernelconcepts.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,7 +35,8 @@
 #include <gpe/pim-categories-ui.h>
 #include <hildon-fm/hildon-widgets/hildon-file-chooser-dialog.h>
 #include <libosso.h>
-#define APPLICATION_DBUS_SERVICE "gpe-calendar"
+#define APPLICATION_DBUS_SERVICE "gpe_calendar"
+#define ICON_PATH "/usr/share/icons/hicolor/26x26/hildon"
 #endif
 
 #include "event-ui.h"
@@ -44,6 +46,7 @@
 #include "week_view.h"
 #include "month_view.h"
 #include "import-vcal.h"
+#include "export-vcal.h"
 #include "gtkdatesel.h"
 
 #include <gpe/pim-categories.h>
@@ -54,23 +57,22 @@
 
 extern gboolean gpe_calendar_start_xsettings (void);
 
-GList *times;
 time_t viewtime;
 gboolean force_today = FALSE;
 gboolean just_new = FALSE;
 
 GtkWidget *main_window, *pop_window;
-GtkWidget *notebook;
+GtkWidget *notebook, *toolbar;
 extern GtkWidget* day_list;
 
 struct gpe_icon my_icons[] = {
-  { "day_view", "day_view" },
-  { "week_view", "week_view" },
-  { "month_view", "month_view" },
-  { "bell", "bell" },
-  { "recur", "recur" },
-  { "bell_recur", "bell_recur" },
-  { "icon", PREFIX "/share/pixmaps/gpe-calendar.png" },
+  { "day_view",   DAY_ICON },
+  { "week_view",  WEEK_ICON },
+  { "month_view", MONTH_ICON },
+  { "bell",        BELL_ICON },
+  { "recur",       RECUR_ICON },
+  { "bell_recur", BELLRECUR_ICON },
+  { "icon", APP_ICON },
   { NULL, NULL }
 };
 
@@ -88,6 +90,22 @@ gint ccpos = 0;
 static guint nr_days[] = { 31, 28, 31, 30, 31, 30, 
 			   31, 31, 30, 31, 30, 31 };
 
+guint
+day_of_week (guint year, guint month, guint day)
+{
+  guint result;
+
+  if (month < 3) 
+    {
+      month += 12;
+      --year;
+    }
+
+  result = day + (13 * month - 27)/5 + year + year/4
+    - year/100 + year/400;
+  return ((result + 6) % 7);
+}
+
 guint
 days_in_month (guint year, guint month)
 {
@@ -114,8 +132,41 @@ time_from_day (int year, int month, int day)
   return selected_time;
 }
 
+/* Call strftime() on the format and convert the result to UTF-8.  Any
+   non-% expressions in the format must be in the locale's character
+   set, since they will undergo UTF-8 conversion.  Careful with
+   translations!  */
+gchar *
+strftime_strdup_utf8_locale (const char *fmt, struct tm *tm)
+{
+  char buf[1024];
+  size_t n;
+
+  buf[0] = '\001';
+  n = strftime (buf, sizeof (buf), fmt, tm);
+  if (n == 0 && buf[0] == '\001')
+    return NULL;		/* Something went wrong */
+
+  return g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+}
+
+/* As above but format string is UTF-8.  */
+gchar *
+strftime_strdup_utf8_utf8 (const char *fmt, struct tm *tm)
+{
+  gchar *sfmt, *sval;
+
+  sfmt = g_locale_from_utf8 (fmt, -1, NULL, NULL, NULL);
+  if (sfmt == NULL)
+    return NULL;		/* Conversion failed */
+  sval = strftime_strdup_utf8_locale (sfmt, tm);
+  g_free (sfmt);
+
+  return sval;
+}
+
 void
-set_time_all_views(void)
+set_time_all_views (void)
 {
   gpointer ds;
   
@@ -216,12 +267,12 @@ new_appointment (void)
   g_timeout_add(1000, do_reset_new, NULL);
   
   appt = new_event (viewtime, 0);
-  g_timeout_add(500, do_insert_text, (gpointer)appt);
+  g_timeout_add(500, (GSourceFunc)(do_insert_text), (gpointer)appt);
   gtk_widget_show (appt);
 }
 
 static void
-set_today(void)
+set_today (void)
 {
   static time_t selected_time;
 
@@ -234,25 +285,35 @@ set_today(void)
   else
     viewtime = selected_time;
   
-  set_time_all_views();
+  set_time_all_views ();
+  day_view_scroll (TRUE);
 }
 
 void
-set_time_and_day_view(time_t selected_time)
+set_time_and_day_view (time_t selected_time)
 {
-  viewtime=selected_time;
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (day_button), TRUE);
+  viewtime = selected_time;
+  gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (day_button), TRUE);
   new_view (day);
-  update_current_view();
+  update_current_view ();
+  day_view_scroll (TRUE);
 }
 
 static void
 button_toggled (GtkWidget *widget, gpointer data)
 {
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+  if (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (widget)))
     new_view (data);
 }
 
+#ifdef IS_HILDON
+static void
+menu_toggled (GtkWidget *widget, gpointer data)
+{
+  gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (data), TRUE);
+}
+#endif
+
 static void
 gpe_cal_exit (void)
 {
@@ -261,21 +322,44 @@ gpe_cal_exit (void)
   gtk_main_quit ();
 }
 
+#ifdef IS_HILDON
+static void
+gpe_cal_fullscreen_toggle (void)
+{
+  static int fullscreen_toggle = TRUE;
+  hildon_appview_set_fullscreen(HILDON_APPVIEW(main_window), fullscreen_toggle);
+  fullscreen_toggle = !fullscreen_toggle;
+}
+
+static void
+toggle_toolbar(GtkCheckMenuItem *menuitem, gpointer user_data)
+{
+  if (gtk_check_menu_item_get_active(menuitem))
+    gtk_widget_show(toolbar);
+  else
+    gtk_widget_hide(toolbar);
+}
+
+static void
+edit_categories (GtkWidget *w)
+{
+  GtkWidget *dialog;
+
+  dialog = gpe_pim_categories_dialog (NULL, FALSE, NULL, NULL);
+  gtk_window_set_transient_for(GTK_WINDOW(dialog), 
+                               GTK_WINDOW(gtk_widget_get_toplevel(w)));
+  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+}
+
+#endif /*IS_HILDON*/
+
 static int 
 import_one_file(gchar *filename)
 {
-  GError *err = NULL;
-  gchar *content;
-  gsize count;
   int result = 0;
-	
-  if (g_file_get_contents(filename,&content,&count,&err))
-    {
-      result = import_vcal(content,count);
-      g_free(content);
-    }
-  else
-    result = -1;
+    
+  result = import_vcal(filename);
+    
   return result;
 }
 
@@ -284,15 +368,24 @@ on_import_vcal (GtkWidget *widget, gpointer data)
 {
   GtkWidget *filesel, *feedbackdlg;
   
+#if IS_HILDON	
+  filesel = hildon_file_chooser_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 
+                                                      GTK_FILE_CHOOSER_ACTION_OPEN);
+  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(filesel), TRUE);
+#else
   filesel = gtk_file_selection_new(_("Choose file"));
   gtk_file_selection_set_select_multiple(GTK_FILE_SELECTION(filesel),TRUE);
-  
+#endif	
   if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK)
     {
       gchar *errstr = NULL;
       int ec = 0, i = 0;
+#ifdef IS_HILDON
+      gchar **files = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(filesel));
+#else		
       gchar **files = 
         gtk_file_selection_get_selections(GTK_FILE_SELECTION(filesel));
+#endif
       gtk_widget_hide(filesel); 
       while (files[i])
         {
@@ -310,18 +403,17 @@ on_import_vcal (GtkWidget *widget, gpointer data)
           i++;  
         }
       if (ec)
-        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(main_window),
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(main_window)),
           GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
           "%s %i %s\n%s",_("Import of"),ec,_("files failed:"),errstr);
       else
-        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(main_window),
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(main_window)),
           GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
           _("Import successful"));
       gtk_dialog_run(GTK_DIALOG(feedbackdlg));
       gtk_widget_destroy(feedbackdlg);
     }
   gtk_widget_destroy(filesel);
-  day_free_lists();
   week_free_lists();
   month_free_lists();
   event_db_refresh();
@@ -341,6 +433,16 @@ notebook_switch_page (GtkNotebook *notebook,
 static gboolean
 main_window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *data)
 {
+#ifdef IS_HILDON
+    /* in hildon there is nothing like control, shift etc buttons */
+    switch (k->keyval)
+      {
+        case GDK_F6:
+        /* toggle button for going full screen */
+        gpe_cal_fullscreen_toggle();
+        break;	
+      }
+#else
   if (k->state & GDK_CONTROL_MASK)
     switch (k->keyval)
       {
@@ -358,6 +460,7 @@ main_window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *data)
           gpe_cal_exit();
         break;	
       }
+#endif /*IS_HILDON*/
   /* ignore if ctrl or alt pressed */    
   if ((k->state & GDK_CONTROL_MASK) 
        || (k->state & GDK_MOD1_MASK))
@@ -380,40 +483,79 @@ main_window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *data)
 static void
 create_app_menu(HildonAppView *appview)
 {
-  GtkMenu *main_menu;
-  GtkWidget *item_appointment, *item_today, *item_day, *item_week,
-    *item_month, *item_import, *item_toolbar;
+  GtkMenu *menu_main = hildon_appview_get_menu(appview);
+  GtkWidget *menu_event = gtk_menu_new();
+  GtkWidget *menu_categories = gtk_menu_new();
+  GtkWidget *menu_view = gtk_menu_new();
+  GtkWidget *menu_tools = gtk_menu_new();
+    
+  GtkWidget *item_event = gtk_menu_item_new_with_label(_("Event"));
+  GtkWidget *item_categories = gtk_menu_item_new_with_label(_("Categories"));
+  GtkWidget *item_view = gtk_menu_item_new_with_label(_("View"));
+  GtkWidget *item_close = gtk_menu_item_new_with_label(_("Close"));
+  GtkWidget *item_add = gtk_menu_item_new_with_label(_("Add new"));
+  GtkWidget *item_delete = gtk_menu_item_new_with_label(_("Delete"));
+  GtkWidget *item_catedit = gtk_menu_item_new_with_label(_("Edit categories"));
+  GtkWidget *item_toolbar = gtk_check_menu_item_new_with_label(_("Show toolbar"));
+  GtkWidget *item_tools = gtk_menu_item_new_with_label(_("Tools"));
+  GtkWidget *item_import = gtk_menu_item_new_with_label(_("Import vCal / ICS"));
+  GtkWidget *item_today = gtk_menu_item_new_with_label(_("Today"));
+  GtkWidget *item_day = gtk_menu_item_new_with_label(_("Day"));
+  GtkWidget *item_week = gtk_menu_item_new_with_label(_("Week"));
+  GtkWidget *item_month = gtk_menu_item_new_with_label(_("Month"));
+  GtkWidget *item_sep = gtk_separator_menu_item_new();
 
-  main_menu = hildon_appview_get_menu(appview);
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_event), menu_event);
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_categories), menu_categories);
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_view), menu_view);
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_tools), menu_tools);
 
-  item_appointment = gtk_menu_item_new_with_label(_("New appointment"));
-  g_signal_connect(G_OBJECT(item_appointment), "activate", G_CALLBACK(new_appointment), NULL);
-  gtk_menu_append(main_menu, item_appointment);
-
-  item_today = gtk_menu_item_new_with_label(_("Today"));
-  g_signal_connect(G_OBJECT(item_today), "activate", set_today, NULL);
-  gtk_menu_append(main_menu, item_today);
-
-  item_import = gtk_menu_item_new_with_label(_("Import"));
+  gtk_menu_append(menu_main, item_event);
+  gtk_menu_append(menu_main, item_categories);
+  gtk_menu_append(menu_main, item_view);
+  gtk_menu_append(menu_main, item_tools);
+  gtk_menu_append(menu_main, item_close);
+  
+  gtk_menu_append(menu_event, item_add);
+  gtk_menu_append(menu_view, item_today);
+  gtk_menu_append(menu_tools, item_import);
+  gtk_menu_append(menu_view, item_day);
+  gtk_menu_append(menu_view, item_week);
+  gtk_menu_append(menu_view, item_month);
+  gtk_menu_append(menu_view, item_sep);
+  gtk_menu_append(menu_view, item_toolbar);
+  gtk_menu_append(menu_categories, item_catedit);
+  
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item_toolbar), TRUE);
+  
+  g_signal_connect(G_OBJECT(item_add), "activate", G_CALLBACK(new_appointment), NULL);
+  g_signal_connect(G_OBJECT(item_today), "activate", G_CALLBACK(set_today), NULL);
   g_signal_connect(G_OBJECT(item_import), "activate", G_CALLBACK(on_import_vcal), NULL);
-  gtk_menu_append(main_menu, item_import);
+  g_signal_connect(G_OBJECT(item_day), "activate", G_CALLBACK(menu_toggled), day_button);
+  g_signal_connect(G_OBJECT(item_week), "activate", G_CALLBACK(menu_toggled), week_button);
+  g_signal_connect(G_OBJECT(item_month), "activate", G_CALLBACK(menu_toggled), month_button);
+  g_signal_connect(G_OBJECT(item_toolbar), "activate", G_CALLBACK(toggle_toolbar), NULL);
+  g_signal_connect(G_OBJECT(item_catedit), "activate", G_CALLBACK(edit_categories), NULL);
+  g_signal_connect(G_OBJECT(item_close), "activate", G_CALLBACK(gpe_cal_exit), NULL);
 
-  gtk_widget_show_all(main_menu);
+  gtk_widget_show_all(GTK_WIDGET(menu_main));
 }
 #endif
 	 
 int
 main (int argc, char *argv[])
 {
-  GtkWidget *vbox, *toolbar;
+  GtkWidget *vbox;
   GdkPixbuf *p;
   GtkWidget *pw;
+  GtkToolItem *item;
+  GtkTooltips *tooltips;
 #ifdef IS_HILDON
   GtkWidget *app, *main_appview;
   osso_context_t *osso_context;
 #endif
 
-  guint hour, skip=0, uid=0;
+  guint skip=0, uid=0;
   int option_letter;
   gboolean schedule_only=FALSE;
   extern char *optarg;
@@ -473,23 +615,8 @@ main (int argc, char *argv[])
       exit (EXIT_SUCCESS);
     }
 	
-  for (hour = 0; hour < 24; hour++)
-    {
-      char buf[32];
-      struct tm tm;
-      time_t t=time(NULL);
-
-      localtime_r (&t, &tm);
-      tm.tm_hour = hour;
-      tm.tm_min = 0;
-      strftime (buf, sizeof(buf), TIMEFMT, &tm);
-      times = g_list_append (times, g_strdup (buf));
-      tm.tm_hour = hour;
-      tm.tm_min = 30;
-      strftime (buf, sizeof(buf), TIMEFMT, &tm);
-      times = g_list_append (times, g_strdup (buf));
-    }
-
+  vcal_export_init();
+    
   vbox = gtk_vbox_new (FALSE, 0);
   notebook = gtk_notebook_new ();
 
@@ -505,6 +632,8 @@ main (int argc, char *argv[])
   main_appview = hildon_appview_new(_("Main"));
   hildon_app_set_appview(HILDON_APP(app), HILDON_APPVIEW(main_appview));
   main_window = main_appview;
+  
+  gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
 #else    
   main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (main_window), _("Calendar"));
@@ -517,8 +646,7 @@ main (int argc, char *argv[])
 
 #ifdef IS_HILDON
   /* Initialize maemo application */
-  osso_context = osso_initialize(
-  APPLICATION_DBUS_SERVICE, "0.1", TRUE, NULL);
+  osso_context = osso_initialize(APPLICATION_DBUS_SERVICE, "0.1", TRUE, NULL);
 
   /* Check that initialization was ok */
   if (osso_context == NULL)
@@ -534,82 +662,133 @@ main (int argc, char *argv[])
   day = day_view ();
   month = month_view ();
   
+  tooltips = gtk_tooltips_new();
+  gtk_tooltips_enable(tooltips);
+  
   toolbar = gtk_toolbar_new ();
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar), GTK_ORIENTATION_HORIZONTAL);
   GTK_WIDGET_UNSET_FLAGS(toolbar, GTK_CAN_FOCUS);
 
-#ifndef IS_HILDON
-  gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
-#else
+#ifdef IS_HILDON
   hildon_appview_set_toolbar(HILDON_APPVIEW(main_appview), GTK_TOOLBAR(toolbar));
+  gtk_widget_show_all(main_appview);
+#else
+  gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
 #endif
   gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);
 
-  pw = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_NEW,
-			    _("New appointment"), _("Tap here to add a new appointment or reminder"),
-			    G_CALLBACK (new_appointment), NULL, -1);
-  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
+  item = gtk_tool_button_new_from_stock(GTK_STOCK_NEW);
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(new_appointment), NULL);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+  gtk_tooltips_set_tip(tooltips, GTK_WIDGET(item), 
+                       _("Tap here to add a new appointment or reminder"), NULL);
+  GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
 
+  if (window_x > 260)
+    {	  
+      item = gtk_separator_tool_item_new();
+      gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(item), FALSE);
+      gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+    }
+
+  pw = gtk_image_new_from_stock (GTK_STOCK_HOME, 
+                                 gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
+  item = gtk_toggle_tool_button_new();
+  gtk_tool_button_set_label(GTK_TOOL_BUTTON(item), _("Today"));
+  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(item), pw);
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(set_today), NULL);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+  gtk_tooltips_set_tip(tooltips, GTK_WIDGET(item), 
+                       _("Switch to today and stay there/return to day selecting."), NULL);
+  GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
+  today_button = GTK_WIDGET(item);    
+    
   if (window_x > 260) 
-    gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+    {	  
+      item = gtk_separator_tool_item_new();
+      gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(item), FALSE);
+      gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+    }
 
-  pw = gtk_image_new_from_stock (GTK_STOCK_HOME, gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
-  today_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar), 
-                                             GTK_TOOLBAR_CHILD_TOGGLEBUTTON, 
-                                             NULL,
-                                             _("Today"), 
-			                                 _("Switch to today and stay there/return to day selecting."), 
-                                             NULL, pw, set_today, NULL);
-  GTK_WIDGET_UNSET_FLAGS(today_button, GTK_CAN_FOCUS);
-
-  if (window_x > 260) 
-    gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
-
-  p = gpe_find_icon_scaled ("day_view", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
+  p = gpe_find_icon_scaled ("day_view", 
+                            gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
   pw = gtk_image_new_from_pixbuf (p);
-  day_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
-					   GTK_TOOLBAR_CHILD_RADIOBUTTON, NULL,
-					   ("Day"), 
-					   _("Tap here to select day-at-a-time view."), NULL,
-					   pw, GTK_SIGNAL_FUNC (button_toggled), day);
-  GTK_WIDGET_UNSET_FLAGS(day_button, GTK_CAN_FOCUS);
-
+  item = gtk_radio_tool_button_new(NULL);
+  gtk_tool_button_set_label(GTK_TOOL_BUTTON(item), _("Day"));
+  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(item), pw);
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(button_toggled), day);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+  gtk_tooltips_set_tip(tooltips, GTK_WIDGET(item), 
+                       _("Tap here to select day-at-a-time view."), NULL);
+  GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
+  day_button = GTK_WIDGET(item);    
+    
+    
   p = gpe_find_icon_scaled ("week_view", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
   pw = gtk_image_new_from_pixbuf (p);
-  week_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
-					    GTK_TOOLBAR_CHILD_RADIOBUTTON, day_button,
-					    _("Week"),
-					    _("Tap here to select week-at-a-time view."), NULL,
-					    pw, GTK_SIGNAL_FUNC (button_toggled), week);
-  GTK_WIDGET_UNSET_FLAGS(week_button, GTK_CAN_FOCUS);
-
+  item = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(item));
+  gtk_tool_button_set_label(GTK_TOOL_BUTTON(item), _("Week"));
+  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(item), pw);
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(button_toggled), week);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+  gtk_tooltips_set_tip(tooltips, GTK_WIDGET(item), 
+                       _("Tap here to select week-at-a-time view."), NULL);
+  GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
+  week_button = GTK_WIDGET(item);    
+  
   p = gpe_find_icon_scaled ("month_view", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
   pw = gtk_image_new_from_pixbuf (p);
-  month_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
-					     GTK_TOOLBAR_CHILD_RADIOBUTTON, week_button,
-					     _("Month"), 
-					     _("Tap here to select month-at-a-time view."), NULL,
-					     pw, GTK_SIGNAL_FUNC (button_toggled), month);
-  GTK_WIDGET_UNSET_FLAGS(month_button, GTK_CAN_FOCUS);
-
+  item = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(item));
+  gtk_tool_button_set_label(GTK_TOOL_BUTTON(item), _("Month"));
+  gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(item), pw);
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(button_toggled), month);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+  gtk_tooltips_set_tip(tooltips, GTK_WIDGET(item), 
+                       _("Tap here to select month-at-a-time view."), NULL);
+  GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
+  month_button = GTK_WIDGET(item);    
+  
+  if (window_x > 260)
+    {	  
+      item = gtk_separator_tool_item_new();
+      gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(item), FALSE);
+      gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+    }
+  
   pw = gtk_image_new_from_stock(GTK_STOCK_OPEN, 
                                 gtk_toolbar_get_icon_size(GTK_TOOLBAR (toolbar)));
-  pw = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-			    _("Import"), _("Open file to import an event from it."), 
-                NULL, pw, G_CALLBACK (on_import_vcal), NULL);
-  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
-
-#ifndef IS_HILDON
-  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
-
-  pw = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_QUIT,
-					_("Tap here to exit the program"), NULL, 
-					G_CALLBACK (gpe_cal_exit), NULL, -1);
-  GTK_WIDGET_UNSET_FLAGS(pw, GTK_CAN_FOCUS);
-#endif
+  item = gtk_tool_button_new(pw, _("Import"));
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(on_import_vcal), NULL);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+  gtk_tooltips_set_tip(tooltips, GTK_WIDGET(item), 
+                       _("Open file to import an event from it."), NULL);
+  GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
 
 #ifdef IS_HILDON
-  create_app_menu(main_appview);
+  pw = gtk_image_new_from_file(ICON_PATH "/qgn_list_gene_bullets.png");
+  item = gtk_tool_button_new(pw, _("Categories"));
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK (edit_categories), 
+                       NULL);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+#else
+  item = gtk_separator_tool_item_new();
+  gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(item), FALSE);
+  gtk_tool_item_set_expand(item, TRUE);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+
+  item = gtk_tool_button_new_from_stock(GTK_STOCK_QUIT);
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(gpe_cal_exit), NULL);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
+  gtk_tooltips_set_tip(tooltips, GTK_WIDGET(item), 
+                       _("Tap here to exit the program"), NULL);
+  GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
+#endif
+
+/* hildon has its special menu, GPE a window icon */
+#ifdef IS_HILDON
+  create_app_menu(HILDON_APPVIEW(main_appview));
+#else
+  gpe_set_window_icon (main_window, "icon");
 #endif
 	 
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
@@ -626,27 +805,28 @@ main (int argc, char *argv[])
                    G_CALLBACK(notebook_switch_page),NULL);
   g_signal_connect (G_OBJECT (main_window), "key_press_event", 
 		    G_CALLBACK (main_window_key_press_event), NULL);
+            
   gtk_widget_add_events (GTK_WIDGET (main_window), 
                          GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
-
+  
   gtk_widget_show (notebook);
 
-  gpe_set_window_icon (main_window, "icon");
-
 #ifdef IS_HILDON  
-  gtk_widget_show_all(app);
+  gtk_widget_show(app);
+  gtk_widget_show(main_appview);
 #endif
   gtk_widget_show (main_window);
   gtk_widget_show (vbox);
-  gtk_widget_show (toolbar);
+  gtk_widget_show_all (toolbar);
 
   gpe_calendar_start_xsettings ();
 
   update_all_views ();
 
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (day_button), TRUE);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (today_button), FALSE);
+  gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (day_button), TRUE);
+  gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (today_button), FALSE);
   new_view (day);
+  day_view_scroll (TRUE);
 
   gtk_main ();
 
