@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001, 2002, 2004 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2001, 2002, 2004, 2005, 2006 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,13 +30,19 @@
 
 static GtkWidget *week_view_draw;
 static struct tm today;
-static guint min_cell_height = 38;
+#ifdef IS_HILDON
+#define REALLY_MIN_CELL_HEIGHT 47
+#else
+#define REALLY_MIN_CELL_HEIGHT 38
+#endif
 static GtkWidget *datesel, *calendar;
 
 static guint time_width, available_width;
 
 static struct render_ctl *c_old;
 
+static GtkWidget *scroller;
+
 struct render_ctl
 {
   struct day_popup popup;
@@ -56,19 +62,21 @@ struct week_day
 
 
 static gint
-draw_expose_event (GtkWidget *widget,
-		   GdkEventExpose *event,
-		   gpointer user_data)
+draw_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
 {
   GtkDrawingArea *darea;
   GdkDrawable *drawable;
   GdkGC *black_gc;
   GdkGC *white_gc;
   GdkGC *blue_gc;
+  GdkGC *yellow_gc;
+  GdkGC *salmon_gc;
   guint max_width;
   guint max_height;
   guint day;
   GdkColor blue;
+  GdkColor yellow;
+  GdkColor salmon;
   GdkColormap *colormap;
   PangoLayout *pl = gtk_widget_create_pango_layout (GTK_WIDGET (widget), NULL);
   PangoLayout *pl_evt = gtk_widget_create_pango_layout (GTK_WIDGET (widget), NULL);
@@ -91,17 +99,25 @@ draw_expose_event (GtkWidget *widget,
   gdk_colormap_alloc_color (colormap, &blue, FALSE, TRUE);
   gdk_gc_set_foreground (blue_gc, &blue);
 
+  yellow_gc = gdk_gc_new (widget->window);
+  gdk_gc_copy (yellow_gc, widget->style->black_gc);
+  gdk_color_parse ("palegoldenrod", &yellow);
+  gdk_colormap_alloc_color (colormap, &yellow, FALSE, TRUE);
+  gdk_gc_set_foreground (yellow_gc, &yellow);
+
+  salmon_gc = gdk_gc_new (widget->window);
+  gdk_gc_copy (salmon_gc, widget->style->black_gc);
+  gdk_color_parse ("light salmon", &salmon);
+  gdk_colormap_alloc_color (colormap, &salmon, FALSE, TRUE);
+  gdk_gc_set_foreground (salmon_gc, &salmon);
+
   darea = GTK_DRAWING_AREA (widget);
   drawable = widget->window;
   white_gc = widget->style->white_gc;
   black_gc = widget->style->black_gc;
   max_width = widget->allocation.width;
   max_height = widget->allocation.height;
-
-  gdk_gc_set_clip_rectangle (black_gc, &event->area);
-  gdk_gc_set_clip_rectangle (blue_gc, &event->area);
-  gdk_gc_set_clip_rectangle (white_gc, &event->area);
-
+  
   for (day = 0; day < 7; day++)
     {
       GSList *iter;
@@ -112,17 +128,21 @@ draw_expose_event (GtkWidget *widget,
   if (wide_mode)
     {
       for (day = 0; day < 7; day++)
-	{
-	  PangoRectangle pr;
-
-	  pango_layout_set_width (pl, max_width * PANGO_SCALE);
-	  pango_layout_set_markup (pl, week_days[day].string, strlen (week_days[day].string));
-	  pango_layout_get_pixel_extents (pl, &pr, NULL);
-	  if (pr.width > day_label_width)
-	    day_label_width = pr.width;
-	}
+        {
+          PangoRectangle pr;
+    
+          pango_layout_set_width (pl, max_width * PANGO_SCALE);
+          pango_layout_set_markup (pl, week_days[day].string, strlen (week_days[day].string));
+          pango_layout_get_pixel_extents (pl, NULL, &pr);
+          if (pr.width > day_label_width)
+            day_label_width = pr.width;
+        }
     }
 
+  gdk_draw_line (drawable, black_gc, 0, 0, 0, max_height);
+  gdk_draw_line (drawable, black_gc, max_width, 0, max_width, max_height);
+  gdk_draw_line (drawable, black_gc, 0, 0, max_width, 0);
+  gdk_draw_line (drawable, black_gc, 0, max_height, max_height, max_height);
   for (day = 0; day < 7; day++)
     {
       guint x, y = week_days[day].y0;
@@ -130,15 +150,17 @@ draw_expose_event (GtkWidget *widget,
       PangoRectangle pr;
 
       if (draw_sep)
-        gdk_draw_line (drawable, black_gc, 0, y, max_width, y);
+        gdk_draw_line (drawable, black_gc, 1, y, max_width, y);
 
-      gdk_draw_rectangle (drawable, white_gc, 
-			  TRUE, 0, y + 1, max_width, max_height);
+      gdk_draw_rectangle (drawable, week_days[day].events ? white_gc :
+                          (day < 5 ? yellow_gc : salmon_gc), 
+			  TRUE, 1, y + 1, max_width, max_height);
 
       pango_layout_set_width (pl, max_width * PANGO_SCALE);
       pango_layout_set_markup (pl, week_days[day].string, strlen (week_days[day].string));
-      pango_layout_get_pixel_extents (pl, &pr, NULL);
-      x = wide_mode ? 4 : max_width - pr.width - 8;
+      pango_layout_get_pixel_extents (pl, NULL, &pr);
+      x = 4;
+//      x = wide_mode ? 4 : max_width - pr.width - 8;
       gtk_paint_layout (widget->style,
 			widget->window,
 			GTK_WIDGET_STATE (widget),
@@ -150,29 +172,30 @@ draw_expose_event (GtkWidget *widget,
 			pl);
 
       if (!wide_mode)
-	y += pr.height + 2;
+        y += pr.height + 2;
 
       if (week_days[day].events)
       {
 	  for (iter = week_days[day].events; iter; iter = iter->next)
 	    {
 	      guint height = 0;
-	      char buf[256];
 	      struct tm tm;
 	      event_t ev = iter->data;
 	      event_details_t evd = event_db_get_details (ev);
-	      int x = wide_mode ? day_label_width + 8 : 2;
+	      int x = wide_mode ? day_label_width + 8 : 8;
 	      
 	      if ((ev->flags & FLAG_UNTIMED) == 0)
 		{
 		  if (ev->mark == FALSE)
 		    {
 		      gchar *buffer;
+
 		      localtime_r (&ev->start, &tm);
-		      strftime (buf, sizeof (buf), "%H:%M", &tm);
-		      buffer = g_locale_to_utf8 (buf, -1, NULL,
-						 NULL, NULL);
-		      pango_layout_set_text (pl_evt, buffer, strlen (buffer));
+
+		      buffer = strftime_strdup_utf8_locale ("%H:%M", &tm);
+		      pango_layout_set_text (pl_evt, buffer, -1);
+		      g_free (buffer);
+
 		      gtk_paint_layout (widget->style,
 					widget->window,
 					GTK_WIDGET_STATE (widget),
@@ -183,17 +206,16 @@ draw_expose_event (GtkWidget *widget,
 					x, y,
 					pl_evt);
 		      
-		      pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
+		      pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
 		      if (height < pr.height)
 			height = pr.height;
 		      ev->mark = TRUE;
-		      g_free (buffer);
 		    }
 		}
 	      
 	      pango_layout_set_width (pl_evt, available_width * PANGO_SCALE);
 	      pango_layout_set_text (pl_evt, evd->summary, -1);
-	      pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
+	      pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
 	      gtk_paint_layout (widget->style,
 				widget->window,
 				GTK_WIDGET_STATE (widget),
@@ -205,7 +227,7 @@ draw_expose_event (GtkWidget *widget,
 				pl_evt);
 
 	      if (height < pr.height)
-		height = pr.height;
+            height = pr.height;
 	      
 	      y += height + 2;
 	    }
@@ -213,7 +235,7 @@ draw_expose_event (GtkWidget *widget,
 
       if (week_days[day].is_today)
         {
-	  gdk_draw_rectangle (drawable, blue_gc, FALSE, 0, 
+          gdk_draw_rectangle (drawable, blue_gc, FALSE, 0,
                               week_days[day].y0, max_width, 
                               week_days[day].y1 - week_days[day].y0);
           gdk_draw_rectangle (drawable, blue_gc, FALSE, 1, 
@@ -225,7 +247,17 @@ draw_expose_event (GtkWidget *widget,
        draw_sep = TRUE;
   }
 
+  {
+    guint y = week_days[6].y1;
+
+    if (draw_sep)
+      gdk_draw_line (drawable, black_gc, 0, y, max_width, y);
+    gdk_draw_rectangle (drawable, white_gc, TRUE, 0, y + 1, max_width, max_height);
+  }
+
   gdk_gc_unref (blue_gc);
+  gdk_gc_unref (yellow_gc);
+  gdk_gc_unref (salmon_gc);
 
   g_object_unref (pl);
   g_object_unref (pl_evt);
@@ -259,6 +291,10 @@ week_view_update (void)
   PangoLayout *pl = gtk_widget_create_pango_layout (GTK_WIDGET (week_view_draw), NULL);
   PangoLayout *pl_evt = gtk_widget_create_pango_layout (GTK_WIDGET (week_view_draw), NULL);
   GSList *iter;
+  guint min_cell_height;
+
+  min_cell_height = MAX (scroller->allocation.height / 7, 
+			 REALLY_MIN_CELL_HEIGHT);
 
   localtime_r (&t, &today);
 
@@ -273,7 +309,6 @@ week_view_update (void)
 
   for (day = 0; day < 7; day++)
     {
-      char buf[32];
       struct week_day *d = &week_days[day];
       struct render_ctl *c = &d->rc;
 
@@ -290,10 +325,10 @@ week_view_update (void)
 		       && tm.tm_mon == today.tm_mon 
 		       && tm.tm_year == today.tm_year);
       week_days[day].is_active = week_days[day].is_today;
-      strftime (buf, sizeof (buf), "<b>%a %d %B</b>", &tm);
+
       if (d->string)
-	g_free (d->string);
-      d->string = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+        g_free (d->string);
+      d->string = strftime_strdup_utf8_utf8 ("<b>%a %d %B</b>", &tm);
 
       c->popup.day = tm.tm_mday;
       c->popup.year = tm.tm_year;
@@ -318,28 +353,21 @@ week_view_update (void)
           /* calculate width required to display times */
           PangoRectangle pr;
           event_t ev = (event_t)iter->data;
-          char buf[32];
-          char *p = buf;
-          size_t l = sizeof (buf) - 1;
-          gchar *s;
+
           if ((ev->flags & FLAG_UNTIMED) == 0)
             {
               if (ev->mark == FALSE)
                 {
-                  size_t n;
+		  gchar *s;
                   localtime_r (&ev->start, &tm);
-                  n = strftime (p, l, "%H:%M", &tm);
-                  p += n;
-                  l -= n;
-                }
-    
-              ev->mark = TRUE;
-              s = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-              pango_layout_set_text (pl_evt, s, strlen (s));
-              pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
-              if (time_width < pr.width)
-                time_width = pr.width;
-              g_free (s);
+		  s = strftime_strdup_utf8_locale ("%H:%M", &tm);
+                  ev->mark = TRUE;
+		  pango_layout_set_text (pl_evt, s, -1);
+		  pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
+		  if (time_width < pr.width)
+		    time_width = pr.width;
+		  g_free (s);
+		}
             }
         }
     }
@@ -355,26 +383,25 @@ week_view_update (void)
 
       pango_layout_set_width (pl, week_view_draw->allocation.width * PANGO_SCALE);
       pango_layout_set_markup (pl, week_days[day].string, strlen (week_days[day].string));
-      pango_layout_get_pixel_extents (pl, &pr, NULL);
+      pango_layout_get_pixel_extents (pl, NULL, &pr);
       height = pr.height + 2;
 
       if (week_days[day].events)
         {
-    
           for (iter = week_days[day].events; iter; iter = iter->next)
             {
               event_t ev = iter->data;
               event_details_t evd = event_db_get_details (ev);
               pango_layout_set_width (pl_evt, available_width * PANGO_SCALE);
               pango_layout_set_text (pl_evt, evd->summary, -1);
-              pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
+              pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
               height += pr.height + 2;
             }
         }
 
-    if (height < min_cell_height)
-      height = min_cell_height;
-
+      if (height < min_cell_height)
+	height = min_cell_height;
+      
       week_days[day].y0 = y;
       y += height;
       week_days[day].y1 = y;
@@ -391,7 +418,7 @@ week_view_update (void)
 static void
 day_changed_calendar (GtkWidget *widget)
 {
-  int day, month, year;
+  unsigned int day, month, year;
   struct tm tm;
 
   localtime_r (&viewtime, &tm);
@@ -567,7 +594,7 @@ week_view_button_press (GtkWidget *widget, GdkEventButton *event, gpointer d)
     
 	      if (c != c_old) 
 		{
-		  pop_window = day_popup (main_window, &c->popup, FALSE);
+		  pop_window = day_popup (gtk_widget_get_toplevel(main_window), &c->popup, FALSE);
 		  c_old = c;
 		}
 	      else 
@@ -601,11 +628,15 @@ week_view (void)
 {
   GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
   GtkWidget *draw = gtk_drawing_area_new ();
-  GtkWidget *scroller = gtk_scrolled_window_new (NULL, NULL);
   gboolean landscape;
   GtkWidget *hbox;
 
-  landscape = gdk_screen_width () > gdk_screen_height () ? TRUE : FALSE;
+  scroller = gtk_scrolled_window_new (NULL, NULL);
+
+  g_signal_connect (G_OBJECT (scroller), "size_allocate", 
+		    G_CALLBACK (week_view_update), NULL);
+
+  landscape = (gdk_screen_width () > gdk_screen_height () && gdk_screen_width () >= 640) ? TRUE : FALSE;
   hbox = gtk_hbox_new (FALSE, 0);
 
   calendar = gtk_calendar_new ();
@@ -614,7 +645,7 @@ week_view (void)
   gtk_calendar_set_display_options (GTK_CALENDAR (calendar), 
 				    GTK_CALENDAR_SHOW_DAY_NAMES 
 				    | (week_starts_monday ? GTK_CALENDAR_WEEK_START_MONDAY : 0));
-
+    
   datesel = gtk_date_sel_new (GTKDATESEL_WEEK);
 
   gtk_widget_show (draw);
@@ -627,7 +658,7 @@ week_view (void)
 
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scroller), draw);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
-				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+				  GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
 
   gtk_box_pack_start (GTK_BOX (vbox), datesel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
@@ -636,12 +667,14 @@ week_view (void)
   if (landscape)
     {
       GtkWidget *sep;
+#ifdef IS_HILDON
+      gtk_widget_set_size_request(calendar, 240, -1);
+#else
       sep = gtk_vseparator_new ();
-
-      gtk_box_pack_start (GTK_BOX (hbox), sep, FALSE, FALSE, 0);
-      gtk_box_pack_start (GTK_BOX (hbox), calendar, FALSE, FALSE, 0);
-
+      gtk_box_pack_start (GTK_BOX (hbox), sep, FALSE, TRUE, 0);
       gtk_widget_show (sep);
+#endif
+      gtk_box_pack_start (GTK_BOX (hbox), calendar, FALSE, TRUE, 0);
       gtk_widget_show (calendar);
     }
 
